#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <liburing.h>
#include <tiffio.h>

#define MAX_TILES 1024

int main()
{
    // 打开 TIFF 文件
    int fd = open("imgs/ortho.tif", O_RDONLY);
    if (fd < 0)
    {
        perror("open");
        return 1;
    }

    TIFF *tif = TIFFOpen("imgs/ortho.tif", "r");
    if (!tif)
    {
        printf("Failed to open TIFF.\n");
        close(fd);
        return 1;
    }

    uint32 tileWidth, tileHeight;
    TIFFGetField(tif, TIFFTAG_TILEWIDTH, &tileWidth);
    TIFFGetField(tif, TIFFTAG_TILELENGTH, &tileHeight);

    uint32 imageWidth, imageHeight;
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imageWidth);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imageHeight);

    uint32 tilesAcross = (imageWidth + tileWidth - 1) / tileWidth;
    uint32 tilesDown = (imageHeight + tileHeight - 1) / tileHeight;
    uint32 totalTiles = tilesAcross * tilesDown;
    if (totalTiles > MAX_TILES)
    {
        printf("Too many tiles (> %d)\n", MAX_TILES);
        TIFFClose(tif);
        close(fd);
        return 1;
    }

    tsize_t *byteCounts = NULL;
    toff_t *offsets = NULL;

    TIFFGetField(tif, TIFFTAG_TILEBYTECOUNTS, &byteCounts);
    TIFFGetField(tif, TIFFTAG_TILEOFFSETS, &offsets);

    // 初始化 io_uring
    struct io_uring ring;
    io_uring_queue_init(totalTiles, &ring, 0);

    // 分配 buffer 指针数组
    char *buffers[MAX_TILES];

    // 提交异步读请求
    for (uint32 i = 0; i < totalTiles; i++)
    {
        buffers[i] = malloc(byteCounts[i]);
        if (!buffers[i])
        {
            perror("malloc");
            // 应该加清理，略
            break;
        }

        struct io_uring_sqe *sqe = io_uring_get_sqe(&ring);
        io_uring_prep_read(sqe, fd, buffers[i], byteCounts[i], offsets[i]);
        io_uring_sqe_set_data(sqe, buffers[i]);
    }

    io_uring_submit(&ring);

    // 等待所有完成
    for (uint32 i = 0; i < totalTiles; i++)
    {
        struct io_uring_cqe *cqe;
        int ret = io_uring_wait_cqe(&ring, &cqe);
        if (ret < 0)
        {
            fprintf(stderr, "io_uring_wait_cqe error: %d\n", ret);
            // 处理错误，略
        }
        if (cqe->res < 0)
        {
            fprintf(stderr, "Async read failed: %d\n", cqe->res);
        }
        else
        {
            // 成功读到 cqe->res 字节数据
            char *data = io_uring_cqe_get_data(cqe);
            // 这里 data 指向对应 tile 的缓冲区
            // 你可以对 data 进一步解码处理
        }
        io_uring_cqe_seen(&ring, cqe);
    }

    // 清理
    for (uint32 i = 0; i < totalTiles; i++)
    {
        free(buffers[i]);
    }
    io_uring_queue_exit(&ring);
    TIFFClose(tif);
    close(fd);

    return 0;
}
