#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <tiffio.h>
#include <limits.h>

int main()
{
    TIFF *tif = TIFFOpen("imgs/ortho.tif", "r");
    if (!tif)
    {
        printf("Failed to open file.\n");
        return 1;
    }

    uint32 tileWidth, tileHeight;
    TIFFGetField(tif, TIFFTAG_TILEWIDTH, &tileWidth);
    TIFFGetField(tif, TIFFTAG_TILELENGTH, &tileHeight);
    printf("Tile size: %dx%d\n", tileWidth, tileHeight);

    uint32 imageWidth, imageHeight;
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imageWidth);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imageHeight);

    uint32 tilesAcross = (imageWidth + tileWidth - 1) / tileWidth;
    uint32 tilesDown = (imageHeight + tileHeight - 1) / tileHeight;
    uint32 totalTiles = tilesAcross * tilesDown;
    printf("Tile count: %d x %d = %d\n", tilesAcross, tilesDown, totalTiles);

    uint16 samplesPerPixel;
    uint16 bitsPerSample;

    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &samplesPerPixel);
    TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &bitsPerSample);

    printf("Samples per pixel: %d\n", samplesPerPixel);
    printf("Bits per sample: %d\n", bitsPerSample);

    uint16 sampleFormat = SAMPLEFORMAT_UINT;
    TIFFGetField(tif, TIFFTAG_SAMPLEFORMAT, &sampleFormat);
    printf("Sample format: %d\n", sampleFormat); // 1=uint, 2=int, 3=float

    tsize_t tileBufSize = TIFFTileSize(tif);
    if (tileBufSize == 0)
    {
        printf("TIFFTileSize returned 0. Possibly not tiled.\n");
        TIFFClose(tif);
        return 1;
    }

    // 分配一个缓冲区用于存放一个 tile 的像素数据（未压缩）
    uint8_t *tileBuf = (uint8_t *)_TIFFmalloc(tileBufSize);
    if (!tileBuf)
    {
        fprintf(stderr, "Failed to allocate tile buffer.\n");
        TIFFClose(tif);
        return 1;
    }

    // 全局最大值初始化
    uint8_t globalMax = 0;

    // 遍历每个 tile 并解码为像素数据
    for (uint32 i = 0; i < totalTiles; i++)
    {
        tsize_t readBytes = TIFFReadEncodedTile(tif, i, tileBuf, tileBufSize);
        if (readBytes == -1)
        {
            fprintf(stderr, "Failed to decode tile %d\n", i);
            continue;
        }

        // 每 tile 最大值初始化
        uint8_t tileMax = 0;

        // 遍历所有像素（只对第一个波段进行最大值统计）
        uint32 numPixels = readBytes / samplesPerPixel;
        for (uint32 px = 0; px < numPixels; px++)
        {
            uint8_t value = tileBuf[px * samplesPerPixel]; // 假设只看第1通道
            if (value > tileMax)
                tileMax = value;
        }

        // 打印 tile 最大值
        printf("Tile %3d: Max = %3d\n", i, tileMax);

        // Reduce 更新全局最大值
        if (tileMax > globalMax)
            globalMax = tileMax;
    }

    printf("\n>>> Global Max Value = %d\n", globalMax);

    _TIFFfree(tileBuf);
    TIFFClose(tif);
    return 0;
}
